# This file contains constants that are used
# to support overriding Kernel functionality.

#--
################################################################################
#                      Copyright (C) 2011 Travis Herrick                       #
################################################################################
#                                                                              #
#                                 \v^V,^!v\^/                                  #
#                                 ~%       %~                                  #
#                                 {  _   _  }                                  #
#                                 (  *   -  )                                  #
#                                 |    /    |                                  #
#                                  \   _,  /                                   #
#                                   \__.__/                                    #
#                                                                              #
################################################################################
# This program is free software: you can redistribute it                       #
# and/or modify it under the terms of the GNU General Public License           #
# as published by the Free Software Foundation,                                #
# either version 3 of the License, or (at your option) any later version.      #
#                                                                              #
# Commercial licensing may be available for a fee under a different license.   #
################################################################################
# This program is distributed in the hope that it will be useful,              #
# but WITHOUT ANY WARRANTY;                                                    #
# without even the implied warranty of MERCHANTABILITY                         #
# or FITNESS FOR A PARTICULAR PURPOSE.                                         #
# See the GNU General Public License for more details.                         #
#                                                                              #
# You should have received a copy of the GNU General Public License            #
# along with this program.  If not, see <http://www.gnu.org/licenses/>.        #
################################################################################
#++

module OverrideKernel

  # This constant is used to indicate whether the application would have
  # been halted by Kernel functions.
  AppState = StateManager.new(:alive, [:alive, :dead])

  def self.method_missing(method, *args, &block)
    if method_list.include?(method.to_s.sub(/^(start|stop)_/, '').to_sym)
      var = '@' + method.to_s.sub(/^(start|stop)_/, '')
      case method.to_s
        when /^start/
          send :instance_variable_set, var.to_sym, true
        when /^stop/
          send :instance_variable_set, var.to_sym, false
        when var[1..-1]
          return nil unless instance_variables.include?(var.to_sym)
          send :instance_variable_get, var.to_sym
      end
    else
      super
    end
  end

  def self.respond_to_missing?(method, include_private)
    if method_list.include?(method.to_s.sub(/^(start|stop)_/, '').to_sym)
      return true
    else
      super
    end
  end

  class << self
    private
    # Returns a list of methods that will be dynamically handled by the module.
    # The following methods will be created:
    #
    #  :start_#{method_name} #=> @method_name = true
    #  :stop_#{method_name}  #=> @method_name = false
    #  :#{method_name}       #=> Returns the value of @method_name
    # ==== Output
    # [Array] An array of base method names.
    def method_list
      [
        :override_exit,
      ]
    end
  end
end
